/*
 * enet.c - 68000 ethernet dependent routines for multi-user telnet
 *
 * Written by Bill Nowicki March 1981
 *
 * Bill Nowicki December 1981
 *	- Put firewall around packet copy
 *
 */

# include "state.h"
# include <pupstatus.h>

# include "m68000.h"
# include "reentrant.h"
# include "vectors.h"
# include "m68enet.h"

 /*
  * offset (IN WORDS!) of the pup from the start of the ethernet packet
  */
# define EthernetOffset 2

char MyHostNumber;

GetNet() { return( getnet( ) ); }

GetHost() { return( MyHostNumber ); }

 /*
  * The ethernet interrupt handler
  */
reentrant(EthernetInterrupt)
 {
   static short buffer[ 1024 ];
   register short  status;
    
    status = EIData0;
    if (status & QueueEmptyBit)
	status = EIData0;	/* Read actual status */

    /*   printf( "Ethernet interrupt" ); */

    while ( !(status & QueueEmptyBit) )
      {
        register short count = status & CountMask;
        register short *pointer = buffer;
	
	if ( count >= sizeof(struct EnPacket)/2 || count <= 10 )
	  {
	     /*
	      * Just throw away packets that are too long
	      * or too short.
	      */
             for (; count--; )
	        status = EIData0;
	     status = EIData0;
	     continue;
	  }

        for (; count--; )		/* move packet to buffer */
	   *pointer++ = EIData0;

        if ( !(status & (OverflowBit|CollisionBit|CRCerrorBit) )
	     && buffer[1]== PUP ) 
           {
		/*
		 * give only good packets to user 
		 */
	     intlevel(6);
	     NetRead( buffer + EthernetOffset );
	     intlevel(1);
	   }
	 status = EIData0;
      }
      
   /* printf( " read receiver "); */

   while ( EIStatus & TSent )
    {
      /*
       * to clear the interrupt??
       */
      status = EIAddress;
      intlevel(6);
      NetDone();
      intlevel(1);
   }

  EIStatus = Init0+IntEnable1+Loopback0+IntLevel1;  
  
  /* printf( " EOEI\n" ); */
 }


InitializeEthernet()
{
  /*
   * initializes the ethernet interface
   * reads host address, sets address filter, and sets up
   * the interrupt vector.
   */
  
    short address, wordbucket;

    EIStatus = Init1+IntEnable0+FilterData0+Loopback0;
    IRQ1Vect = (int)EthernetInterrupt;

    for (address=0; address<256; address++)	/* Clear Address Filter */
		EIAddress = address;

      /* 
       * read our Address from the switches,
       * and enable both it and broadcast (host 0) packets
       */
    address = EIAddress;			

    EIStatus = Init1+IntEnable0+FilterData1+Loopback0;
    EIAddress = address;
    EIAddress = 0;	
    MyHostNumber = address;
    EIStatus = Init0+IntEnable1+Loopback0+IntLevel1; 

    wordbucket = EIData0; /* Synchronize data port lookahead */
}


NetWrite( packet, immediateDestination )
 struct PupPacket *packet;
 unsigned char immediateDestination;
 {
  /*
   * Write the PUP packet to the network,
   * using the given immediate destination
   */
   
   short firstWord;
   register int roundlen = roundup( packet->PupLength );
	/*
	 * number of WORDs in buffer. Ethernet packet has the
	 * encapsulated PUP plus src/dst word and packet type.
	 */
   register short count = (roundlen >> 1 ) + EthernetOffset;	
   register short *buffer = (short *) packet;

   firstWord = (MyHostNumber&0377) | (immediateDestination << 8);

    EIData0 = count;
    EIData0 = firstWord;
    EIData0 = PUP;

    for ( count -= 3; count-- ;) EIData0 = *buffer++;
	
    EIData0 = checksum( packet, roundlen-2 );

/*    EIStatus = Init0+IntEnable1+Loopback0+CollisionTest0+IntLevel1; */
  }
 
